<?php
/**
 * @package Tetonline
 */
class TetOnline 
{
    private $url;
    private $allow_final_consumer;
    private $api_key;
    public $emaildocument;
    private $api_docref;
    private $doc_type;

    public function __construct()
    {
        $this->allow_final_consumer = Configuration::get('TETONLINE_ALLOW_FINAL_CONSUMER');
        $this->doc_type             = Configuration::get('TETONLINE_DOCS'); 
        $this->emaildocument        = Configuration::get('TETONLINE_EMAIL');
        $this->api_docref           = Configuration::get('TETONLINE_REF');
        $this->url                  = Configuration::get('TETONLINE_API_URL');
        $this->api_key              = Configuration::get('TETONLINE_API_KEY');
    }

    public function basic_curl($url)
    {
        $c = curl_init();
        curl_setopt_array($c, array(
            CURLOPT_URL => $this->url.$url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 100,
            CURLOPT_TIMEOUT => 180,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_HTTPHEADER => array("Authorization: Bearer ".$this->api_key))
        );

        $response = curl_exec($c);
        $httpcode = (int)curl_getinfo($c, CURLINFO_HTTP_CODE);
        curl_close($c);

        if ($httpcode == 200) 
        {
            return json_decode($response);
        } 
        else 
        {
            return Array();
        }
    }

    public function curl_direct_api($crud, $store, $direct_api)
    {
        $c = curl_init();
        curl_setopt_array($c, array(
            CURLOPT_URL => $direct_api ? $this->url.'direct-api/'.$store.'/'.$crud : $this->url.$store.'/'.$crud,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 100,
            CURLOPT_TIMEOUT => 180,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_HTTPHEADER => array("Authorization: Bearer ".$this->api_key),
            CURLOPT_POSTFIELDS => json_encode(array('StoreID' => $store)))
        );

        $response = curl_exec($c);
        $httpcode = (int)curl_getinfo($c, CURLINFO_HTTP_CODE);
        curl_close($c);

        if ($httpcode == 200) 
        {
            return json_decode($response);
        } 
        else 
        {
            return Array();
        }
    }

    public function curl_direct_api_filter($crud, $store, $direct_api, $filter)
    {
        $c = curl_init();
        curl_setopt_array($c, array(
            CURLOPT_URL => $direct_api ? $this->url.'direct-api/'.$store.'/'.$crud : $this->url.$store.'/'.$crud,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 100,
            CURLOPT_TIMEOUT => 180,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_HTTPHEADER => array("Authorization: Bearer ".$this->api_key),
            CURLOPT_POSTFIELDS => json_encode($filter))
        );

        $response = curl_exec($c);
        $httpcode = (int)curl_getinfo($c, CURLINFO_HTTP_CODE);
        curl_close($c);

        if ($httpcode == 200) 
        {
            return json_decode($response);
        } 
        else 
        {
            return Array();
        }
    }

    public function curl_api($store, $body, $file=false)
    {
        $c = curl_init();
        curl_setopt_array($c, array(
            CURLOPT_URL => $this->url.$store,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 100,
            CURLOPT_TIMEOUT => 180,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_HTTPHEADER => array("Authorization: Bearer ".$this->api_key),
            CURLOPT_POSTFIELDS => json_encode($body))
        );

        $response = curl_exec($c);
        $httpcode = (int)curl_getinfo($c, CURLINFO_HTTP_CODE);
        curl_close($c);
    
        if ($httpcode == 200) 
        {
            if ($file) 
            {
                return $response;
            }
            return json_decode($response);
        } 
        else 
        {
            return array();
        }
    }

    public function about()
    {
        return $this->basic_curl('about');
    }

    public function getCompany()
    {
        return $this->curl_direct_api('read', 'company', true);
    }

    private function getCmpAddresses($cmpid)
    {
        return $this->curl_direct_api_filter(
            'read', 'company_addresses', true,
            array('StoreID'=> 'company_addresses', 'filter'=> array(
                array('property'=> 'CMP_ADR_COMPANYID', 'opertaor'=>'eq', 'value'=> $cmpid, 'perm_filter'=> true))
            )
        );
    }

    public function getProducts()
    {
        return $this->curl_direct_api(
            'read', 'articles', true
        );
    }

    public function getTaxes()
    {
        return $this->curl_direct_api(
            'read', 'article_taxes', true
        );
    }

    public function getAllPrices()
    {
        return $this->curl_direct_api('read', 'article_priceslist', true);
    }

    private function getPrices($art_id)
    {
        return $this->curl_direct_api_filter(
            'read', 'article_priceslist', true,
            array('StoreID'=> 'article_priceslist', 'filter'=> 
                array(
                    array('property'=> 'PRSL_ART_ID', 'operator'=> 'eq', 'value'=> $art_id, 'perm_filter'=> true)
                )
            )
        );
    }

    public function getFamilies()
    {
        return $this->curl_direct_api(
            'read', 'article_families', true
        );
    }

    private function getPreferences()
    {
        return $this->curl_api('preferencias', array('action'=> 'load'));
    }

    public function getImageBlob($art_id)
    {
        return $this->curl_direct_api_filter(
            'getblob', 'articles', true,
            array('filter'=>
                array(
                    array('property'=> 'ART_ID', 'operator'=> 'eq', 'value'=> $art_id, 'perm_filter'=> true)
                ),
                'returnfields'=> ["ART_FOTO"]
            )
        );
    }

    private function clientExists($nif)
    {
        return $this->curl_api('entidades',
            array('action'=> 'check_dup_nif', 'nif'=> $nif, 'ent_id'=> '') 
        );
    }

    private function validateNif($nif, $country)
    {
        return $this->curl_api('entidades',
            array('action'=> 'valid_nif', 'nif'=> $nif, 'country'=> $country) 
        );
    }

    private function getTetCountries()
    {
        return $this->curl_direct_api('read', 'entities_countries', true);
    }

    private function getAddressTypes()
    {
        return $this->curl_direct_api('read', 'address_types_combo', true);
    }

    public function generateDocPdf($doc_id, $doc_type)
    {
        return $this->curl_api('doprintdoc', 
            array('action'=> 'doprintdoc', 'language'=> 'pt', 'doccab_id'=> $doc_id, 'doccab_code_id'=> $doc_type, 'doccab_doctipo'=> 'D', 'model_id'=> 0)
        );
    }

    public function getDocPdf($uuid_doc, $doc_name)
    {
        return $this->curl_api('get_print', 
            array("id"=> $uuid_doc, "ext"=> "pdf", "name"=> $doc_name, "filename"=> $doc_name.".pdf", "down"=> 1), true
        );
    }

    private function getClient($nif, $t_country)
    {
        return $this->curl_direct_api_filter(
            'read', 'entities', true,
            array('StoreID'=> 'entities', 'filter'=> array(
                    array('property'=> 'ENT_IS_CUSTOMER', 'operator'=> 'eq', 'value'=> true, 'perm_filter'=> true),
                    array('property'=> 'ENT_CONTRIB', 'operator'=> 'eq', 'value'=> $nif, 'perm_filter'=> true),
                    array('property'=> 'ENT_NACION_ID', 'operator'=> 'eq', 'value'=> $t_country, 'perm_filter'=> true),
                    array('property'=> 'ENT_STATE', 'operator'=> 'eq', 'value'=> 1, 'perm_filter'=> true)
                )
            )
        );
    }

    private function getActiveSerie($docType)
    {
        return $this->curl_direct_api_filter(
            'read', 'docum_series', true,
            array('StoreID'=> 'docum_series', 'filter'=> 
                array(
                    array('property'=> 'DOCSRS_STATE', 'operator'=> 'eq', 'value'=> true, 'perm_filter'=> true),
                    array('property'=> 'DOCSRS_DOC_TYPE', 'operator'=> 'eq', 'value'=> $docType, 'perm_filter'=> true),
                    array('property'=> 'DOCSRS_ACT_STATE', 'operator'=> 'eq', 'value'=> 'A', 'perm_filter'=> true),
                    array('property'=> 'DOCSRS_ACT_YEAR_PREFIX', 'operator'=> 'eq', 'value'=> date('y'), 'perm_filter'=> true),
                    array('property'=> 'DOCSRS_TYPE', 'operator'=> 'eq', 'value'=> 1, 'perm_filter'=> true)
                )
            )
        );
    }

    private function getProduct($ref)
    {
        return $this->curl_direct_api_filter(
            'read', 'articles', true,
            array('StoreID'=> 'articles', 'filter'=> 
                array(
                    array('property'=> 'ART_REFERENCE', 'operator'=> 'eq', 'value'=> $ref, 'perm_filter'=> true)
                )
            )
        );
    }

    public function getEmailTemplates($entID, $docID)
    {
        return $this->curl_api(
            'emdocumentos',
            array('action'=> 'verifyEmail', 'cliid'=> $entID, 'docid'=> $docID) 
        );
    }

    public function sendDocEmail($args)
    {
        return $this->curl_api(
            'senddocemail', 
            array('action'=> 'senddocemail', 'docid'=> $args['docID'], 'tplid'=> $args['templateID'])
        );
    }

    private function updatePrice($data)
    {
        return $this->curl_direct_api_filter(
            'update', 'article_priceslist', true,
            array('article_priceslist_items'=> array($data), 'StoreID'=> 'article_priceslist')
        );
    }

    private function createFamily($data)
    {
        return $this->curl_direct_api_filter(
            'create', 'article_families', true,
            array('article_families_items'=> array($data), 'StoreID'=> 'article_families')
        );
    }

    private function createProduct($data)
    {
        return $this->curl_direct_api_filter(
            'create', 'articles', true,
            array('articles_items'=> array($data), 'StoreID'=> 'articles')
        );
    }

    private function insertClient($nif, $address, $shipping, $t_country, $fiscal_addr, $unload_addr)
    {
        $data = array(
            'action'=> 'saveentitie',
            'data'=> array(
                "ENT_IS_CUSTOMER"=> true,
                "ENT_IS_SUPPLIER"=> false,
                "ENT_IS_SALESMAN"=> false,
                "ENT_IS_BANK"=> false,
                "ENT_DATA_CONSENT"=> true,
                "ENT_ACRONYM"=> "",
                "ENT_NAME"=> $address->firstname.' '.$address->lastname,
                "ENT_CONTRIB"=> $nif,
                "ENT_NACION_ID"=> $t_country,
                "moradas"=> array(
                    array(
                        "ENT_ADDRESS1_ID"=> true,
                        "ENT_ADDRESS2_ID"=> false,
                        "ENT_ADDRESS_LOAD"=> false,
                        "ENT_ADDRESS_UNLOAD"=> false,
                        "ENT_ADR_ADDRESS"=> $address->address1.' '.$address->address2,
                        "ENT_ADR_COUNTRY"=> $t_country,
                        "ENT_ADR_LOCALITY"=> $address->city,
                        "ENT_ADR_REGION"=> $address->city,
                        "ENT_ADR_TYPE"=> $fiscal_addr,
                        "ENT_ADR_ZIPCODE"=> $address->postcode
                    )
                )
            )
        );

        if (count((array)$shipping) > 0)
        {
            $data['data']['moradas'][] = array(
                    "ENT_ADDRESS1_ID"=> false,
                    "ENT_ADDRESS2_ID"=> false,
                    "ENT_ADDRESS_LOAD"=> false,
                    "ENT_ADDRESS_UNLOAD"=> true,
                    "ENT_ADR_ADDRESS"=> $shipping->address1.' '.$shipping->address2,
                    "ENT_ADR_COUNTRY"=> $t_country,
                    "ENT_ADR_LOCALITY"=> $shipping->city,
                    "ENT_ADR_REGION"=> $shipping->city,
                    "ENT_ADR_TYPE"=> $unload_addr,
                    "ENT_ADR_ZIPCODE"=> $shipping->postcode
            );
        }
        return $this->curl_api('entidades', $data);
    }

    private function updateClient($nif, $email, $address, $shipping_address, $cli, $fiscal_addr)
    {
        $data = array(
            //'ENT_CONTRIB'=> $nif,
            'ENT_PHONE'=> $address->phone,
            'ENT_EMAIL'=> $email,
            'ENT_COMMENTS'=> 'API->PRESTASHOP',
            'ENT_RECVERSION'=> $cli->ENT_RECVERSION,
            'ENT_ID'=> $cli->ENT_ID
        );

        $name = trim($address->firstname.' '.$address->lastname);

        if (($name != '') && (strtoupper($name) != strtoupper($cli->ENT_NAME)))
        {
            $data['ENT_NAME'] = $name;
        }

        $result = $this->curl_direct_api_filter(
            'update', 'entities', true,
            array('entities_items'=> array($data), 'StoreID'=> 'entities')
        );

        $new_adr = trim($address->address1.' '.$address->address2);
        if (($result->success === true) && ($new_adr !== ''))
        {
            $addr = $this->curl_direct_api_filter(
                'read', 'entities_addresses', true,
                array('StoreID'=> 'entities_addresses', 'filter'=> array(
                    array('property'=> 'ENT_ADR_ENT_ID', 'value'=> $cli->ENT_ID, 'operator'=> 'eq', 'perm_filter'=> true),
                    array('property'=> 'ENT_ADR_TYPE', 'value'=> $fiscal_addr, 'operator'=> 'eq', 'perm_filter'=> true))
                )
            );
      
            if ($addr->success === true)
            {
                if ($addr->totalCount > 0)
                {
                    $upd_addr = $this->curl_direct_api_filter(
                        'update', 'entities_addresses', true,
                        array('StoreID'=> 'entities_addresses', 'entities_addresses_items'=> array( 
                            array('ENT_ADR_ID'=> $addr->entities_addresses_items[0]->ENT_ADR_ID,
                                'ENT_ADR_ADDRESS'=> $address->address1.' '.$address->address2,
                                'ENT_ADR_LOCALITY'=> $address->city,
                                'ENT_ADR_REGION'=> $address->city,
                                'ENT_ADR_RECVERSION'=> $addr->entities_addresses_items[0]->ENT_ADR_RECVERSION
                            )
                        ))
                    );
                }
            }
        }
        return $result;
    }

    private function insertDocument($fields)
    {
        $docum = array(
            "action" => "savedocumcab", 
            "fields" => array(
                'documCab'=> $fields
            )
        );
        return $this->curl_api('emdocumentos', $docum);
    }

    private function findCountrie($tet_countries, $order)
    {
        if (($tet_countries->success === true) && ($tet_countries->totalCount > 0))
        {
            for ($i = 0; $i < $tet_countries->totalCount; $i++)
            {
                $country = $tet_countries->entities_countries_items[$i];
                if ($order->iso_code !== '') 
                {
                    if ($country->NAT_CODE === $order->iso_code)
                    {
                        return $country->NAT_ID;
                    }
                }
            }
        }
        return 0;
    }

    private function findSCountrie($tet_countries, $shipping)
    {
        for ($i = 0; $i < $tet_countries->totalCount; $i++)
        {
            $country = $tet_countries->entities_countries_items[$i];
            if ($shipping->iso_code !== '') 
            {
                if ($country->NAT_CODE === $shipping->iso_code)
                {
                    return $country->NAT_ID;
                }
            }
        }
        return 0;
    }

    private function findFiscalAddr($addr)
    {
        if (($addr->success === true) && ($addr->totalCount > 0))
        {
            for ($i = 0; $i < $addr->totalCount; $i++)
            {
                $tmp_addr = $addr->address_types_combo_items[$i];
                if ($tmp_addr->DESCR_DESCR == 'text.fiscal_address')
                {
                    return $tmp_addr->DESCR_CODE;
                }
            }
        }
        return 0;
    }

    private function findShippAddr($addr)
    {
        if (($addr->success === true) && ($addr->totalCount > 0))
        {
            for ($i = 0; $i < $addr->totalCount; $i++)
            {
                $tmp_addr = $addr->address_types_combo_items[$i];
                if ($tmp_addr->DESCR_DESCR == 'text.delivery_pickup_address')
                {
                    return $tmp_addr->DESCR_CODE;
                }
            }
        }
        return 0;
    }

    public function familySearch($families, $search)
    {
        if (($families->success === true) && ($families->totalCount > 0))
        {
            foreach($families->article_families_items as $fam_key => $fam_value)
            {
                if (strtoupper($fam_value->ARTFAM_DESCR) === strtoupper($search))
                {
                    return $fam_value->ARTFAM_ID;
                }
            }
        }
        return false;
    }

    private function getCompanyAddr($company, $type)
    {
        if (($company->success === true) && ($company->totalCount > 0))
        {
            for ($i = 0; $i < $company->totalCount; $i++)
            {
                $tmp = $company->company_addresses_items[$i];
                if ($tmp->CMP_ADR_TYPE === $type)
                {
                    return $tmp;
                }
            }
        }
        return array();
    }

    private function findTax($taxes, $tax)
    {
        if (($taxes->success === true) && ($taxes->totalCount > 0))
        {
            for ($i = 0; $i < $taxes->totalCount; $i++)
            {
                $tmp_tx = $taxes->article_taxes_items[$i];
                if ($tmp_tx->TAX_VALUE == $tax)
                {
                    return $tmp_tx;
                }
            }
        }
        return array();
    }

    public function getCliPrefs()
    {
        $cli_prefs = '';
        $preferencias = $this->getPreferences();
        if (($preferencias->success === true) && !empty($preferencias->data[0]->PRP_PROPVALUE))
        {
            $cli_prefs = json_decode($preferencias->data[0]->PRP_PROPVALUE);
        }
        return $cli_prefs;
    }

    public function generateInvoice($order_id)
    {
        $order = new Order($order_id);
        $t_country = 0;
        $v_nif = false;
        $client_data = array();
        $new_product = false;
        $new_product_params = array();

        if (!$order) {die("error, no order");}
        
        $customer = new Customer($order->id_customer);
        $invoice_address = new Address($order->id_address_invoice);
        $country = new Country($invoice_address->id_country);
        $delivery_address = new Address($order->id_address_delivery);
        $country_delivery = new Country($delivery_address->id_country);

        $tet_countries = $this->getTetCountries();
        $tet_add_types = $this->getAddressTypes();
    
        $fiscal_addr = $this->findFiscalAddr($tet_add_types);
        $unload_addr = $this->findShippAddr($tet_add_types);
        $t_country   = $this->findCountrie($tet_countries, $country);
        $s_country   = $this->findSCountrie($tet_countries, $country_delivery);
        $families    = $this->getFamilies();
        $taxes       = $this->getTaxes();
        $tet_series  = $this->getActiveSerie($this->doc_type);
        $tet_prefs   = $this->getCliPrefs();

        if ((($tet_series->success === true) && ($tet_series->totalCount === 0)) || ($tet_series->success === false))
        {
            return json_encode(array('msg'=> 'Não existem séries ativas para criar o documento!'));
        }

        $final_consumer = false;
        $client_ok      = false;
     
        $nif = $invoice_address->dni;
        
        if ((trim($nif) != '') && ($t_country === 182)) {
            $tmp_v_nif = $this->validateNif($nif, $t_country);
            if ($tmp_v_nif->success == true)
            {
                $v_nif = $tmp_v_nif->valid_nif;
            }
        }

        if (trim($nif) != '')
        {          
            $result = $this->clientExists($nif);
            if (($result->success === true) && ($result->data[0]->CONTRIB === 1))
            {
                $t_cli = $this->getClient($nif, $t_country);
                if ($t_cli->success === true)
                {
                    $result = $this->updateClient($nif, $customer->email, $invoice_address, $delivery_address, $t_cli->entities_items[0], $fiscal_addr);
                    if ($result->success)
                    {
                        $clientok = true;
                        $upd_cli = $this->getClient($nif, $t_country);
                        $client_data = $upd_cli->entities_items[0];
                    }
                }
            }
            else if (($result->success === true) && ((int)$result->data[0]->CONTRIB > 1))
            {
                return json_encode(array('msg'=> 'Existem mais do que um cliente com o mesmo NIF!'));
            }
            else if ($result->success === false)
            {
                return json_encode(array('msg'=> 'Ocorreu um erro ao verificar cliente!'));
            }
            else
            {
                if (($country->iso_code == 'PT'))
                {
                    if ($v_nif)
                    {
                        $result = $this->insertClient($nif, $invoice_address, $delivery_address, $t_country, $fiscal_addr, $unload_addr);
                        if ($result->success === true)
                        {
                            $clientok = true;
                            $client_data = $result->data[0];
                        }
                        else
                        {
                            return json_encode(array('msg'=> 'Ocorreu um erro ao criar cliente!'));                                   
                        }
                    }
                    else
                    {
                        return json_encode(array('msg'=> 'Para criar um cliente Português tem que ter um NIF válido!'));   
                    } 
                }
                else
                {
                    if ($order->billing_country != '')
                    {
                        $result = $this->insertClient($nif, $invoice_address, $delivery_address, $t_country, $fiscal_addr, $unload_addr);
                        if ($result->success === true)
                        {
                            $clientok = true;
                            $client_data = $result->data[0];
                        }
                        else
                        {
                            return json_encode(array('msg'=> 'Ocorreu um erro ao criar cliente!'));                                    
                        }
                    }     
                }
            }	
        }
        else
        {
            $final_consumer = true;
        } 

        if ((($final_consumer) && $this->allow_final_consumer) && ((int)$this->doc_type !== 66)) //SE FOR FINAL CONSUMER E DIFERENTE DE FATURA SIMPLIFICADA NÃO DÁ
        {
            if ((int)$this->doc_type == 1)
            {
                $type = 'Fatura';
            }
            else if ((int)$this->doc_type == 14)
            {
                $type = 'Encomenda';
            }
            return json_encode(array('msg'=> 'Não é possível exportar o documento do tipo '.$type.' para um consumidor final.'));
        }
        elseif ((!$this->allow_final_consumer) && ($final_consumer == true))
        {
            return json_encode(array('msg'=> 'Não é possível exportar o documento para um consumidor final.'));
        }

        $docref = '';
        if (trim($this->api_docref) != '')
        {
            $docref = trim($this->api_docref);
        }
        
        $company = $this->getCompany();
        $cmp_addr = '';
        if (($company->success !== true))
        {
            return json_encode(array('msg'=> 'Não foi possível obter os dados da empresa.'));
        }
        else
        {
            $cmp_addr = $this->getCmpAddresses($company->company_items[0]->CMP_COMPANYID);
            if ($cmp_addr->success !== true)
            {
                return json_encode(array('msg'=> 'Não é possível obter as moradas da empresa.'));
            }
        }

        $cmp_fiscal = $this->getCompanyAddr($cmp_addr, $fiscal_addr);
        $cmp_load = $this->getCompanyAddr($cmp_addr, $unload_addr);
        
        $cli_name   = '';
        $cli_nif    = '';
        $cli_nacion = 182;
        $cli_ent_id = 0;
        if ($final_consumer)
        {
            $cli_name = 'Consumidor final';
            $cli_nif  = '999999990'; 
        }
        elseif ($clientok && (!$final_consumer))
        {

            $cli        = $client_data;
            $cli_name   = $cli->ENT_NAME;
            $cli_nif    = $cli->ENT_CONTRIB;
            $cli_nacion = $cli->ENT_NACION_ID;
            $cli_ent_id = $cli->ENT_ID;
        }
        
        $doc_date = date('Y-m-d\TH:i:s.000\Z');

        $documCab = array(
            "DOCCAB_ENT_NAME" => $cli_name, 
            "DOCCAB_ENT_CONTRIB" => $cli_nif, 
            "DOCCAB_ENT_NACION_ID" => $cli_nacion, 
            "DOCCAB_CODE_ID" => (int)$this->doc_type,
            "DOCCAB_AV_PERIOD" => 0, 
            "DOCCAB_SERIES_ID" => $tet_series->docum_series_items[0]->DOCSRS_ID, 
            "DOCCAB_HASHCONTROL_NUMBER" => "", 
            "DOCCAB_HASHCONTROL_SERIES" => "", 
            "DOCCAB_DATE" => $doc_date, 
            "DOCCAB_EXPIR_DATE" => $doc_date, 
            "DOCCAB_CAE_CODE" => "", 
            "DOCCAB_PAYCOND_ID" => 0, 
            "DOCCAB_SALESMAN_ID" => 0, 
            "DOCCAB_YOUR_DOC_REF" => $docref.$order->id.'_'.$order->reference, 
            "DOCCAB_DISPMODE_ID" => 0, 
            "DOCCAB_PAYMODE_ID" => 0, 
            "DOCCAB_IVAINC" => false, 
            "DOCCAB_STOCKMOVE" => true, 
            "DOCCAB_STATE_ID" => 1, 
            "DOCCAB_CMP_ADDRESS" => $cmp_fiscal->CMP_ADR_ADDRESS, 
            "DOCCAB_CMP_LOCALITY" => $cmp_fiscal->CMP_ADR_LOCALITY, 
            "DOCCAB_CMP_ZIPCODE" => $cmp_fiscal->CMP_ADR_ZIPCODE, 
            "DOCCAB_CMP_COUNTRY" => $cmp_fiscal->CMP_ADR_COUNTRY,
            "DOCCAB_CMP_REGION"=> $cmp_fiscal->CMP_ADR_REGION,  
            "DOCCAB_ENT_ADDRESS" => $invoice_address->address1.' '.$invoice_address->address2, 
            "DOCCAB_ENT_LOCALITY" => $invoice_address->city, 
            "DOCCAB_ENT_ZIPCODE" => $invoice_address->postcode, 
            "DOCCAB_ENT_COUNTRY" =>  $t_country,
            "DOCCAB_CMP_REGION" => $invoice_address->city, 
            "DOCCAB_LOAD_ADDRESS" => $cmp_load->CMP_ADR_ADDRESS, 
            "DOCCAB_LOAD_LOCALITY" => $cmp_load->CMP_ADR_LOCALITY, 
            "DOCCAB_LOAD_ZIPCODE" => $cmp_load->CMP_ADR_ZIPCODE, 
            "DOCCAB_LOAD_COUNTRY" => $cmp_load->CMP_ADR_COUNTRY,
            "DOCCAB_LOAD_REGION" =>$cmp_load->CMP_ADR_REGION, 
            "DOCCAB_UNLOAD_ADDRESS" => $delivery_address->address1.' '.$delivery_address->address2, 
            "DOCCAB_UNLOAD_LOCALITY" => $delivery_address->city, 
            "DOCCAB_UNLOAD_ZIPCODE" => $delivery_address->postcode, 
            "DOCCAB_UNLOAD_COUNTRY" => $s_country,
            "DOCCAB_UNLOAD_REGION"=> $delivery_address->city,
            "DOCCAB_VEHICLE_REGISTR" => "", 
            "DOCCAB_COMMENTS" => "PRESTASHOP Encomenda Nº ".$order->id.' ('.$order->reference.')', 
            "DOCCAB_ENT_ID" => $cli_ent_id, 
            "DOCCAB_DCC_SRS_ID" => $tet_series->docum_series_items[0]->DOCSRS_ID, 
            "DOCCAB_MBREF_HASREF" => false, 
            "DOCCAB_MBREF_ENT" => "", 
            "DOCCAB_MBREF_REFERENCE" => "", 
            "DOCCAB_MBREF_ID" => "", 
            "DOCCAB_MBREF_AUTO_PAYED" => false, 
            "DOCCAB_LOAD_DATE" => $doc_date, 
            "DOCCAB_ENT_TYPE" => "cli", 
            "ENT_ADDRESS1_ID" => null,
            "DOCCAB_REC_SRS_ID" => null
        );

        $items = $order->getOrderDetailList();
        foreach($items as $item)
        {
            unset($tax);
		    $order_detail_tax = $this->getDetailTax($item['id_order_detail']);
     
            if (is_array($order_detail_tax) && !empty($order_detail_tax))
            {
                $tax = new Tax((int)$order_detail_tax[0]['id_tax'], true);
            }

            $tet_taxes = $this->findTax($taxes, $tax->rate);

            if (is_null($tet_taxes))
            {
                return json_encode(array('msg'=> 'Não é possível exportar o documento. Erro ao carregar as taxas.'));
            }

            $product = new Product((int)$item['product_id']);
            
            if (!empty($product->reference))
            {
                $product_exists = $this->getProduct($product->reference);
            }
            else
            {
                return json_encode(array('msg'=> 'Não é possível exportar o documento. Erro ao verificar produto, falta de referência.'));   
            }

            if (($product_exists->success === true) && ($product_exists->totalCount > 1))
            {
                return json_encode(array('msg'=> 'Não é possível exportar o documento. Existe mais do que um producto com a mesma referência.'));
            }
            else if ($product_exists->success === false)
            {
                return json_encode(array('msg'=> 'Não é possível exportar o documento. Erro ao carregar informação do produto.'));
            }
            else if (($product_exists->success === true) && ($product_exists->totalCount === 0))
            {
                if (!empty($product->reference))
                {
                    $new_product_params = array(
                        'ART_DESCR'=> $product->name[(int)Configuration::get('PS_LANG_DEFAULT')],
                        'ART_SHORT_DESCR'=> substr($product->name[(int)Configuration::get('PS_LANG_DEFAULT')], 0, 29),
                        'ART_REFERENCE'=> $product->reference,
                        'ART_STOCK_MOVE'=> 1,
                        'ART_IVA_TXR_ID'=> $tet_taxes->TAX_ID,
                        'ART_ARTTYP_ID'=> 'P',
                        'ART_STATE'=> 1,
                        'ART_ARTFAM_ID'=> 0,
                        'ART_COMMENTS'=> 'PRESTASHOP->API',
                        'ART_PRODUCER'=> 0
                    );
                    $new_product = true;
                }
                else
                {
                    return json_encode(array('msg'=> 'Não é possível exportar o documento. Erro ao criar o produto por falta de referência.'));
                }
                
            }
            if ($new_product)
            {
                $product_exists = $this->createProduct($new_product_params);

                if ($product_exists->success === false)
                {
                    return json_encode(array('msg'=> 'Erro, ao inserir Produto.'));
                }
                else
                {
                    //update the price for the new product
                    $price = $this->getPrices($product_exists->articles_items[0]->ART_ID);
                    if ($price->success === true) 
                    {
                        $p = $price->article_priceslist_items[0];
                        $param = array(
                            'PRICESLIST_ID'=> $p->PRICESLIST_ID,
                            'PRSL_COIN'=> (float)$item['product_price'],
                            'PRSL_COIN_VAT'=> ($item['product_price'] * (1 + ($tet_taxes->TAX_VALUE / 100))),
                            'PRSL_RECVERSION'=> $p->PRSL_RECVERSION
                        );
                        $this->updatePrice($param);
                    }
                }
            }

            $discount = 0;
            if ($item['reduction_percent'] > 0) $discount = $item['reduction_percent']; 
            if ($item['reduction_amount_tax_excl'] > 0) $discount = ($item['reduction_amount_tax_excl'] / ($item['product_price'])) * 100;

            $sub_total = ($item['product_price'] * (float)$item['product_quantity']);

            $prod_desc = $product->name[(int)Configuration::get('PS_LANG_DEFAULT')];
            $prod_info = $product_exists->articles_items[0];
            $line = array(
                "DOCDET_ART_ID" => $prod_info->ART_ID,
                "DOCDET_TXR_ID" => 0,
                "DOCDET_ART_MOV_STOCK" => ($prod_info->ART_STOCK_MOVE === 1) ? true : false,
                "DOCDET_IVA_ISE" => "",
                "DOCDET_IVA_ISE_CODE" => $tet_taxes->TAX_IVAEXR_ID,
                "DOCDET_IVA_TAX_CODE" => $tet_taxes->TAX_CODE,
                "DOCDET_IVA_TAX_REGION" => $tet_taxes->TAX_REGION,
                "DOCDET_ART_REF" => $product->reference,
                "DOCDET_DESCR" => $prod_desc,
                "DOCDET_UNIT_PRICE_DOC" => floatval($item['product_price']),
                "DOCDET_QUANT" => (float)$item['product_quantity'],
                "DOCDET_UNIT_CODE" => "",
                "DOCDET_UNIT_ROUND" => 0,
                "DOCDET_DESC_PERCENT" => floatval($discount),
                "DOCDET_IVA_TAX" => $tet_taxes->TAX_VALUE,
                "DOCDET_RET_TAX" => 0,
                "DOCDET_SUBTOTAL_DOC" => $sub_total,
                "DOCDET_REF_DOCASS_ID" => 0,
                "DOCDET_REF_DOCCAB_ID" => 0,
                "DOCDET_REF_REASON" => "",
                "IMPORTED" => false,
                "DOCDET_ID" => 0,
            );
            $documCab['documDet'][] = $line;
        }

        if ($order->total_shipping > 0)
        {
            $ship_taxes = $this->findTax($taxes,  $order->carrier_tax_rate);
            //if ($api_prices_with_vat) $shipping += $shipping_tax;
            // NAO INSERE PREÇOS COM IVA
            $documCab['documDet'][] = array(
                "DOCDET_ART_ID" => 0,
                "DOCDET_TXR_ID" => 0,
                "DOCDET_ART_MOV_STOCK" => false,
                "DOCDET_IVA_ISE" => '',
                "DOCDET_IVA_ISE_CODE" => $ship_taxes->TAX_IVAEXR_ID,
                "DOCDET_IVA_TAX_CODE" => $ship_taxes->TAX_CODE,
                "DOCDET_IVA_TAX_REGION" => $ship_taxes->TAX_REGION,
                "DOCDET_ART_REF" => '',
                "DOCDET_DESCR" => __('Portes de Envio'),
                "DOCDET_UNIT_PRICE_DOC" => floatval($order->total_shipping_tax_excl),
                "DOCDET_QUANT" => 1,
                "DOCDET_UNIT_CODE" => '',
                "DOCDET_UNIT_ROUND" => 0,
                "DOCDET_DESC_PERCENT" => 0,
                "DOCDET_IVA_TAX" => $ship_taxes->TAX_VALUE,
                "DOCDET_RET_TAX" => 0,
                "DOCDET_SUBTOTAL_DOC" => floatval($order->total_shipping_tax_excl),
                "DOCDET_REF_DOCASS_ID" => 0,
                "DOCDET_REF_DOCCAB_ID" => 0,
                "DOCDET_REF_REASON" => '',
                "IMPORTED" => false,
                "DOCDET_ID" => 0,
            );
        }

        $doc_ins = $this->insertDocument($documCab);

        if (($doc_ins->success === false)) {
            return json_encode(array('msg'=> 'Erro ao criar documento.'));
        }

        $document_key = $doc_ins->docid.'_'.$this->doc_type.'_'.$cli_ent_id;
       
        $doc_old_key = Db::getInstance()->getRow('SELECT `_tetonline_document_id`
					FROM `'._DB_PREFIX_.'orders`
					WHERE `id_order` = \''.pSQL($order->id).'\'');

        if (is_array($doc_old_key) && !empty($doc_old_key['_tetonline_document_id']))
        {
            if ($doc_old_key['_tetonline_document_id'] != $document_key)
            {
                Db::getInstance()->execute("UPDATE "._DB_PREFIX_."orders SET _tetonline_document_id = '".$document_key."' WHERE id_order = '".$order->id."'");
            }
        }
        else
        {
            Db::getInstance()->execute("UPDATE "._DB_PREFIX_."orders SET _tetonline_document_id = '".$document_key."' WHERE id_order = '".$order->id."'");
        }
        return json_encode(array('msg'=> 'A encomenda foi exportada com sucesso.'));
    }
    
    private function getDetailTax($id_order_detail)
    {	
        $query = new DbQuery();
        $query->select('tax.*');
        $query->from('order_detail_tax', 'tax');
        $query->join('LEFT JOIN `'._DB_PREFIX_.'order_detail` od ON (tax.`id_order_detail` = od.`id_order_detail`)');
        $query->where('od.`id_order_detail` = '.(int)$id_order_detail);

        return Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($query);
    }

    public function __destruct()
    {
        unset($this->url);
        unset($this->familySearchallow_final_consumer);
        unset($this->api_key);
        unset($this->emaildocument);
        unset($this->api_docref);
        unset($this->doc_type);
    }
}