<?php

require_once(dirname(__DIR__, 3).'/config/config.inc.php');
require_once(dirname(__DIR__).'/classes/API/TetOnline.php');

function copyImg($id_entity, $id_image = null, $blob, $entity = 'products', $regenerate = true)
{
    if ($blob->success && (trim($blob->returnfields[0]->ART_FOTO) !== ''))
    {
        $string64 = explode('||', trim($blob->returnfields[0]->ART_FOTO));
    }
    else
    {
        return false;
    }

    $tempfile = tempnam(_PS_TMP_IMG_DIR_, 'ps_import');
    $watermark_types = explode(', ', Configuration::get('WATERMARK_TYPES'));

    $tmp_f = dirname( __FILE__ ).'/'.$string64[1];

    $f = fopen($tmp_f, 'w');
    fwrite($f, base64_decode($string64[0]));
    fclose($f);

    switch($entity)
    {
        default:
        case 'products':
            $image_obj = new Image($id_image);
            $path = $image_obj->getPathForCreation();
        break;
        case 'categories':
            $path = _PS_CAT_IMG_DIR_.(int)$id_entity;
        break;
        case 'manufacturers':
            $path = _PS_MANU_IMG_DIR_.(int)$id_entity;
        break;
        case 'suppliers':
            $path = _PS_SUPP_IMG_DIR_.(int)$id_entity;
        break;
    }
    //$url = str_replace(' ', '%20', trim($url));
    // Evaluate the memory required to resize the image: if it's too much, you can't resize it.
    if (!ImageManager::checkImageMemoryLimit($tmp_f))
    {
        return false;
    }

   if (Tools::copy($tmp_f, $tempfile))
   {
        //copy($tmp_f, $tempfile);
        $ext = explode('.', $string64[1]);
        ImageManager::resize($tempfile, $path.'.'.$ext[1]);
        $image_types = ImageType::getImagesTypes($entity);

        if ($regenerate)
        {
            foreach($image_types as $image_type)
            {
                ImageManager::resize($tempfile, $path.'-'.stripslashes($image_type['name']).'.'.$ext[1], $image_type['width'], $image_type['height']);
                if (in_array($image_type['id_image_type'], $watermark_types))
                {
                    Hook::exec('actionWatermark', array('id_image'=> $id_image, 'id_product'=> $id_entity));
                }
            }
        }
   } 
   else
   {
        unlink($tmp_f);
        unlink($tempfile);
        return false;
    }
    unlink($tmp_f);
    unlink($tempfile);
    return true;
}

function importProducts()
{
    $total = 0;
    
    if ((trim(Configuration::get('TETONLINE_API_URL')) == '') || (trim(Configuration::get('TETONLINE_API_KEY')) == ''))
    {
        echo json_encode(array('msg'=> 'É necessário configurar a chave e/ou o url da API do TetOnline.'));
        return;
    }
    
    $tet = new TetOnline();
    $company = $tet->getCompany();

    if (($company->success !== true))
	{
		echo json_encode(array('msg'=> 'Não foi possivel fazer ligação ao programa de faturação.'));
        return;
	}

    $products = $tet->getProducts();
	$prices = $tet->getAllPrices();
	$taxes = $tet->getTaxes();
	$families = $tet->getFamilies();

    $num_prod = 0;
	if ($products->success == true)
    {
        $num_prod = intval($products->totalCount) - 1;
    }
    $created_products = $updated_products = 0;
	
    set_time_limit(0);
    if ($num_prod > 0)
    {
        $aLangs = Language::getLanguages(false);
        
        for ($i = 0; $i <= $num_prod; $i++)
        {
            $tproduct = $products->articles_items[$i];
			$price = findPrice($prices, $tproduct->ART_ID);
			$txs = findTax($taxes, $tproduct->ART_IVA_TXR_ID);

            if ($txs === -1)
            {
                echo json_encode(array('msg'=> 'Não foi possivel fazer a importação, existem produtos sem taxa de IVA.'));
                return;
            }

            $id_product = (int)Db::getInstance()->getValue('SELECT id_product FROM '._DB_PREFIX_.'product WHERE reference = \''.pSQL($tproduct->ART_REFERENCE).'\'');
     
            if (!$id_product)
            {
                $product = new Product();
                $product->reference = $tproduct->ART_REFERENCE;
                $created_products++;
            }
            else 
            {
                $product = new Product((int)$id_product);
                $updated_products++;
            }

            Tools::displayAsDeprecated();
            $tax = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow('
                SELECT t.`id_tax`
                FROM `'._DB_PREFIX_.'tax` t
                LEFT JOIN `'._DB_PREFIX_.'tax_lang` tl ON (t.id_tax = tl.id_tax)
                WHERE t.`rate` = '.(float)($txs->TAX_VALUE));
            $tax_id = $tax ? (int)($tax['id_tax']): false;

            if (!$tax_id)
            {
                $tax = new Tax();
                foreach($aLangs as $lang)
                {
                    $tax->name[$lang['id_lang']] = 'IVA'." ".float($txs->TAX_VALUE).'%';
                }
                $tax->rate = $txs->TAX_VALUE;
                $tax->active = 1;
                $tax->save();
            }
            else
            {
                $tax = new Tax($tax_id);
            }

            $name_rule = 'TET_'.$txs->TAX_VALUE.'%';

            $tax_rule_group = TaxRulesGroup::getIdByName($name_rule);
            if (!$tax_rule_group)
            {
                $trg = new TaxRulesGroup();
                $trg->name = $name_rule;
                $trg->active = 1;
                $trg->save();

                TaxRule::deleteByGroupId($trg->id);
                $countries = Country::getCountries((int)Configuration::get('PS_LANG_DEFAULT'), false, false, false);
                foreach($countries as $country)
                {
                    $tax_rule = new TaxRule();
                    $tax_rule->id_tax_rules_group = $trg->id;
                    $tax_rule->id_country = $country['id_country'];
                    $tax_rule->id_tax = $tax->id;
                    $tax_rule->save();
                }
            }
            else
            {
                $trg = new TaxRulesGroup($tax_rule_group);
            }

            $price = number_format((float)floatval($price), 9, '.', '');

            $product->id_tax_rules_group = $trg->id;
            $product->price = (float)$price;
            $product->active = $tproduct->ART_STATE;

            foreach($aLangs as $lang)
            {
                $product->name[$lang['id_lang']] = $tproduct->ART_DESCR;
                $product->description[$lang['id_lang']] = $tproduct->ART_DESCR;
                $product->description_short[$lang['id_lang']] = $tproduct->ART_SHORT_DESCR;
                $product->link_rewrite[$lang['id_lang']] = Tools::link_rewrite($tproduct->ART_DESCR);

                $product->meta_description[$lang['id_lang']] = htmlspecialchars($tproduct->ART_SHORT_DESCR);
                $product->meta_title[$lang['id_lang']] = htmlspecialchars($tproduct->ART_DESCR);
            }
            $product->save();

            if ($tproduct->ART_EXISTING_STOCK > 0)
            {
                StockAvailable::setQuantity($product->id, 0, $tproduct->ART_EXISTING_STOCK);
            }

            /*if (trim($tproduct->ART_FOTO) !== '')
            {
                $image = new Image();
                //ver se imagem existe?
                if (!$image->hasImages((int)Configuration::get('PS_LANG_DEFAULT'), $product->id))
                {
                    $shops = Shop::getShops(true, null, true);
                    $image->id_product = $product->id;
                    $image->position = Image::getHighestPosition($product->id) + 1;
                    $image->cover = true;

                    if (($image->validateFields(false, true)) === true &&
                        ($image->validateFieldsLang(false, true)) === true && $image->add())
                    {
                        $image->associateTo($shops);

                        if (!copyImg($product->id, $image->id, $tet->getImageBlob($tproduct->ART_ID), 'products', false))
                        {
                            $image->delete();
                        }
                    }
                }
            }*/

            if (!isset($product->date_add) || empty($product->date_add))
            {
                $product->date_add = date('Y-m-d H:i:s');
            }
            $product->date_upd = date('Y-m-d H:i:s');
            $product->save();

            $fam = findFamilie($families, $tproduct->ART_ARTFAM_ID);
            if ($fam)
            {
                $id_category = (int)Db::getInstance()->getValue('SELECT c.id_category 
                        FROM `'._DB_PREFIX_.'category` c
                        '.Shop::addSqlAssociation('category', 'c').'
                        WHERE tetonline_ref = \''.pSQL($tproduct->ART_ARTFAM_ID).'\'  
                        GROUP BY c.id_category ');

                if (intval($id_category))
                {
                    $category = new Category((int)$id_category);
                    if (!Category::categoryExists($category->id_parent))
                    {
                        $category->id_parent = Category::getRootCategory()->id;
                    }
                }
                else
                {
                    $category = new Category();
                    $category->id_parent = Category::getRootCategory()->id;
                }

                foreach($aLangs as $lang)
                {
                    $category->name[$lang['id_lang']] = $fam;
                    $category->description[$lang['id_lang']] = $fam;
                    $category->link_rewrite[$lang['id_lang']] = Tools::link_rewrite($fam);
                }
                $category->save();

                Db::getInstance()->execute("UPDATE "._DB_PREFIX_."category SET tetonline_ref = '".$tproduct->ART_ARTFAM_ID ."' WHERE id_category = '".$category->id."'");

                $product->addToCategories($category->id);
                $product->id_category_default = $category->id;
                $product->save();
            }
        }
    }
    else
    {
        echo json_encode(array('msg' => 'Não foram encontrados produtos para importação.'));
        return;
    }
    $total = (int)$updated_products + (int)$created_products;
    echo json_encode(array('msg' => 'Importaçao Finalizada com sucesso! '.$total.' Artigo(s) sincronizado(s)'));
    return;
}

function findPrice($prices, $art_id)
{
	for ($i = 0; $i < $prices->totalCount; $i++)
	{
		if ($art_id === $prices->article_priceslist_items[$i]->PRSL_ART_ID)
		{
			return $prices->article_priceslist_items[$i]->PRSL_COIN;
		}
	}
    return 0;
}

function findTax($taxes, $tax_id)
{
	for ($i = 0; $i < $taxes->totalCount; $i++)
	{
		if ($tax_id === $taxes->article_taxes_items[$i]->TAX_ID)
		{
			return $taxes->article_taxes_items[$i];
		}
	}
    return -1;
}

function findFamilie($families, $art_familie_id)
{
	for ($i = 0; $i < $families->totalCount; $i++)
	{
		if ($art_familie_id === $families->article_families_items[$i]->ARTFAM_ID)
		{
			return $families->article_families_items[$i]->ARTFAM_DESCR;
		}
	}
    return '';
}

$req = $_POST;
$action = $req['action'];

switch ($action) 
{
    case 'import_products': {
        echo importProducts();
        break;
    }

}

